<template>
  <basic-container>
    <avue-crud ref="crud"
               v-model="form"
               :before-close="beforeClose"
               :before-open="beforeOpen"
               :data="data"
               :option="option"
               :permission="permissionList"
               :table-loading="loading"
               @row-del="rowDel"
               @row-update="rowUpdate"
               @row-save="rowSave"
               @search-change="searchChange"
               @search-reset="searchReset"
               @selection-change="selectionChange"
               @row-click="clickRowChange"
               @current-change="currentChange"
               @size-change="sizeChange"
               @refresh-change="refreshChange"
               @on-load="onLoad"
               @tree-load="treeLoad">
      <template slot="menuLeft">
        <el-button v-if="this.permission.menu.menu_delete"
                   icon="el-icon-delete"
                   plain
                   size="small"
                   type="danger"
                   @click="handleDelete">删 除
        </el-button>
        <el-button v-if="this.permission.menu.menu_clone"
                   icon="el-icon-connection"
                   plain
                   size="small"
                   type="success"
                   @click="cloneMenuButton">从其他菜单克隆按钮
        </el-button>
      </template>
      <template slot="menu" slot-scope="scope">
        <el-button
          v-if="userInfo.role_name.includes('admin') && scope.row.category === 1"
          icon="el-icon-circle-plus-outline"
          size="small"
          type="text"
          @click.stop="handleAdd(scope.row,scope.index)"
        >新增子项
        </el-button>
      </template>
      <template slot="source"
                slot-scope="{row}">
        <div style="text-align:center">
          <i :class="row.source"/>
        </div>
      </template>
    </avue-crud>
    <!-- 从其他菜单克隆按钮 -->
    <el-dialog :visible.sync="cloneOtherMenuButtons.cloneButtonSettingBox"
               append-to-body
               style="height: 110vh; margin-top: -10vh;"
               title="从其他菜单克隆按钮"
               width="1100px"
               @close="cleanCloneSelections">
      <avue-crud ref="cloneCrud"
                 :data="cloneOtherMenuButtons.cloneData"
                 :option="cloneOtherMenuButtons.cloneOption"
                 :table-loading="cloneOtherMenuButtons.cloneLoading"
                 @search-change="searchCloneChange"
                 @search-reset="searchCloneReset"
                 @selection-change="selectionCloneChange"
                 @row-click="clickRowCloneChange"
                 @refresh-change="refreshCloneChange"
                 @on-load="cloneOnLoad"
                 @tree-load="treeLoad">
        <template slot="source" slot-scope="{row}">
          <div style="text-align:center">
            <i :class="row.source"/>
          </div>
        </template>
      </avue-crud>
      <div slot="footer" class="dialog-footer">
        <el-button type="primary" @click="cloneButtonsToMenu">保 存</el-button>
        <el-button @click="cloneOtherMenuButtons.cloneButtonSettingBox = false">取 消</el-button>
      </div>
    </el-dialog>
  </basic-container>
</template>

<script>
import {getLazyList, remove, update, add, getMenu, cloneMenuButton} from "@/api/system/menu";
import {mapGetters} from "vuex";
import iconList from "@/config/iconList";
import func from "@/util/func";
import {getMenuTree} from "@/api/system/menu";

export default {
  data() {
    return {
      form: {},
      query: {},
      loading: true,
      selectionList: [],
      parentId: 0,
      page: {
        pageSize: 10,
        currentPage: 1,
        total: 0,
      },
      option: {
        height: "auto",
        lazy: true,
        columnBtn: false,
        tip: false,
        simplePage: true,
        searchShow: true,
        searchMenuSpan: 6,
        dialogWidth: "60%",
        tree: true,
        border: true,
        index: true,
        selection: true,
        viewBtn: true,
        menuWidth: 300,
        dialogClickModal: false,
        highlightCurrentRow: true, //行选中时高亮
        column: [
          {
            label: "菜单名称",
            prop: "name",
            search: true,
            width:220,
            rules: [
              {
                required: true,
                message: "请输入菜单名称",
                trigger: "blur"
              }
            ]
          },
          {
            label: "路由地址",
            prop: "path",
            rules: [
              {
                required: true,
                message: "请输入路由地址",
                trigger: "blur"
              }
            ]
          },
          {
            label: "上级菜单",
            prop: "parentId",
            type: "tree",
            dicData: [],
            hide: true,
            addDisabled: false,
            props: {
              label: "title"
            },
            rules: [
              {
                required: false,
                message: "请选择上级菜单",
                trigger: "click"
              }
            ]
          },
          {
            label: "菜单图标",
            prop: "source",
            type: "icon",
            slot: true,
            iconList: iconList,
            rules: [
              {
                required: true,
                message: "请输入菜单图标",
                trigger: "click"
              }
            ]
          },
          {
            label: "菜单编号",
            prop: "code",
            search: true,
            rules: [
              {
                required: true,
                message: "请输入菜单编号",
                trigger: "blur"
              }
            ]
          },
          {
            label: "菜单类型",
            prop: "category",
            type: "radio",
            dicData: [
              {
                label: "菜单",
                value: 1
              },
              {
                label: "按钮",
                value: 2
              }
            ],
            hide: true,
            rules: [
              {
                required: true,
                message: "请选择菜单类型",
                trigger: "blur"
              }
            ]
          },
          {
            label: "菜单别名",
            prop: "alias",
            search: true,
            rules: [
              {
                required: true,
                message: "请输入菜单别名",
                trigger: "blur"
              }
            ]
          },
          {
            label: "新窗口",
            prop: "isOpen",
            type: "radio",
            disabled: false,
            display: false,
            dicData: [
              {
                label: "否",
                value: 1
              },
              {
                label: "是",
                value: 2
              }
            ],
            value: 1,
            rules: [
              {
                required: true,
                message: "请选择新窗口打开",
                trigger: "blur"
              }
            ]
          },
          {
            label: "菜单排序",
            prop: "sort",
            type: "number",
            rules: [
              {
                required: true,
                message: "请输入菜单排序",
                trigger: "blur"
              }
            ]
          },
          {
            label: "路由缓存",
            prop: "keepAlive",
            type: "switch",
            dicData: [
              {
                label: "否",
                value: "false"
              },
              {
                label: "是",
                value: "true"
              }
            ],
            hide: true
          },
          {
            label: "菜单备注",
            prop: "remark",
            type: "textarea",
            span: 24,
            minRows: 2,
            hide: true
          }
        ]
      },
      data: [],
      // 从其他菜单克隆按钮
      cloneOtherMenuButtons: {
        cloneButtonSettingBox: false,
        selectCloneButtons: [],
        cloneOption: {
          height: "auto",
          menu: false,
          header: false,
          addBtn: false,
          lazy: true,
          tip: false,
          simplePage: true,
          searchShow: true,
          searchMenuSpan: 6,
          dialogWidth: "60%",
          tree: true,
          border: true,
          index: true,
          selection: true,
          columnBtn: false,
          searchShowBtn: false,
          menuWidth: 300,
          dialogClickModal: false,
          highlightCurrentRow: true, //行选中时高亮
          column: [
            {
              label: "菜单名称",
              prop: "name",
              search: true,
            },
            {
              label: "路由地址",
              prop: "path",
            },
            {
              label: "上级菜单",
              prop: "parentId",
              type: "tree",
              hide: true,
              addDisabled: false,
              props: {
                label: "title"
              },
            },
            {
              label: "菜单图标",
              prop: "source",
              type: "icon",
              slot: true,
              iconList: iconList,
            },
            {
              label: "菜单编号",
              prop: "code",
              search: true,
            },
            {
              label: "菜单类型",
              prop: "category",
              type: "radio",
              dicData: [
                {
                  label: "菜单",
                  value: 1
                },
                {
                  label: "按钮",
                  value: 2
                }
              ],
            },
            {
              label: "菜单别名",
              prop: "alias",
              search: true,
            },
            {
              label: "新窗口",
              prop: "isOpen",
              type: "radio",
              dicData: [
                {
                  label: "否",
                  value: 1
                },
                {
                  label: "是",
                  value: 2
                }
              ],
            },
            {
              label: "菜单排序",
              prop: "sort",
              type: "number",
            },
            {
              label: "路由缓存",
              prop: "keepAlive",
              type: "switch",
              dicData: [
                {
                  label: "否",
                  value: "false"
                },
                {
                  label: "是",
                  value: "true"
                }
              ],
            },
            {
              label: "菜单备注",
              prop: "remark",
              type: "textarea",
              span: 24,
              minRows: 2,
            }
          ]
        },
        cloneLoading: false,
        cloneData: [],
        cloneQuery: {},
        clonePage: {
          pageSize: 10,
          currentPage: 1,
          total: 0,
        },
      }
    };
  },
  watch: {
    'form.category'() {
      const category = func.toInt(this.form.category);
      this.$refs.crud.option.column.filter(item => {
        if (item.prop === "path") {
          item.rules[0].required = category === 1;
        }
        if (item.prop === 'isOpen') {
          item.disabled = category === 2;
        }
      });
    },
  },
  computed: {
    // mapGetters: 辅助函数仅仅将store 中的 getter 映射到局部计算属性
    ...mapGetters(["userInfo", "permission"]),
    permissionList() {
      return {
        addBtn: this.vaildData(this.permission.menu.menu_add, false),
        viewBtn: this.vaildData(this.permission.menu.menu_view, false),
        delBtn: this.vaildData(this.permission.menu.menu_delete, false),
        editBtn: this.vaildData(this.permission.menu.menu_edit, false)
      };

    },
    ids() {
      let ids = [];
      this.selectionList.forEach(ele => {
        ids.push(ele.id);
      });
      return ids.join(",");
    }
  },
  methods: {
    /** 从其他菜单克隆按钮 */
    cloneMenuButton() {
      if (this.selectionList.length != 1 || this.selectionList[0].category === 2) {
        this.$message({
          type: "warning",
          message: "请选择一条非按钮类型的数据!"
        });
        return;
      }
      this.cloneOtherMenuButtons.cloneButtonSettingBox = true;
    },
    /** 克隆按钮到选中的菜单下 */
    cloneButtonsToMenu() {
      if (this.selectionList.length != 1) {
        this.$message({
          type: "warning",
          message: "要克隆按钮的菜单信息已丢失，请重新选择!"
        });
        return;
      }
      if (this.cloneOtherMenuButtons.selectCloneButtons.length <= 0) {
        this.$message({
          type: "warning",
          message: "请选择要克隆按钮数据!"
        });
        return;
      }
      this.$confirm("确定将选中的按钮克隆到【" + this.selectionList[0].name + "】菜单中?", {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "info"
      })
        .then(() => {
          let buttonIds = [];
          let flag = true;
          this.cloneOtherMenuButtons.selectCloneButtons.forEach(function (item) {
            if (item.category != 2) {
              flag = false;
            } else {
              buttonIds.push(item.id);
            }
          });
          // 选择了非按钮类型的数据
          if (!flag) {
            this.$message({
              type: "warning",
              message: "只能选择按钮类型的表格行!"
            });
            return;
          }
          // console.log(buttonIds);
          return cloneMenuButton({"menuId": this.selectionList[0].id, "buttonIds": buttonIds});
        })
        .then(res => {
          this.$message({
            type: res.data.success ? "success" : "error",
            message: res.data.msg,
          });
          this.cloneOtherMenuButtons.cloneButtonSettingBox = false;
          // 数据回调进行刷新
          this.onLoad(this.page);
        });
    },
    /** 选中被克隆的按钮时触发 */
    selectionCloneChange(list) {
      this.cloneOtherMenuButtons.selectCloneButtons = list;
      this.$refs.cloneCrud.setCurrentRow(this.cloneOtherMenuButtons.selectCloneButtons[list.length - 1]);
    },
    clickRowCloneChange(row) {
      this.cloneOtherMenuButtons.selectCloneButtons.push(row);
      this.$refs.cloneCrud.setCurrentRow(row);
      this.$refs.cloneCrud.toggleRowSelection(row); //选中当前行
    },
    /** 关闭对话框时清空选中的列表 */
    cleanCloneSelections() {
      this.cloneOtherMenuButtons.selectCloneButtons = [];
      this.$refs.cloneCrud.toggleSelection();
    },
    cloneOnLoad(page, params = {}) {
      this.cloneOtherMenuButtons.cloneLoading = true;
      getLazyList(this.parentId, Object.assign(params, this.cloneOtherMenuButtons.cloneQuery)).then(res => {
        this.cloneOtherMenuButtons.cloneData = res.data.data;
        this.cloneOtherMenuButtons.cloneLoading = false;
      });
    },
    searchCloneChange(params, done) {
      this.cloneOtherMenuButtons.cloneQuery = params;
      this.parentId = '';
      this.cloneOtherMenuButtons.clonePage.currentPage = 1;
      this.cloneOnLoad(this.cloneOtherMenuButtons.clonePage, params);
      done();
    },
    searchCloneReset() {
      this.cloneOtherMenuButtons.cloneQuery = {};
      this.parentId = 0;
      this.cloneOnLoad(this.cloneOtherMenuButtons.clonePage);
    },
    refreshCloneChange() {
      this.onLoad(this.cloneOtherMenuButtons.clonePage, this.cloneOtherMenuButtons.cloneQuery);
    },

    initData() {
      getMenuTree().then(res => {
        const column = this.findObject(this.option.column, "parentId");
        column.dicData = res.data.data;
      });
    },
    handleAdd(row) {
      this.parentId = row.id;
      const column = this.findObject(this.option.column, "parentId");
      column.value = row.id;
      column.addDisabled = true;
      this.$refs.crud.rowAdd();
    },
    rowSave(row, done, loading) {
      add(row).then((res) => {
        // 获取新增数据的相关字段
        const data = res.data.data;
        row.id = data.id;
        this.$message({
          type: "success",
          message: "操作成功!"
        });
        // 数据回调进行刷新
        done(row);
      }, error => {
        window.console.log(error);
        loading();
      });
    },
    rowUpdate(row, index, done, loading) {
      update(row).then(() => {
        this.$message({
          type: "success",
          message: "操作成功!"
        });
        // 数据回调进行刷新
        this.onLoad()
        done(row);
      }, error => {
        window.console.log(error);
        loading();
      });
    },
    rowDel(row, index, done) {
      this.$confirm("确定将选择数据删除?", {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning"
      })
        .then(() => {
          return remove(row.id);
        })
        .then(() => {
          this.$message({
            type: "success",
            message: "操作成功!"
          });
          // 数据回调进行刷新
          done(row);
        });
    },
    handleDelete() {
      if (this.selectionList.length === 0) {
        this.$message.warning("请选择至少一条数据");
        return;
      }
      this.$confirm("确定将选择数据删除?", {
        confirmButtonText: "确定",
        cancelButtonText: "取消",
        type: "warning"
      })
        .then(() => {
          return remove(this.ids);
        })
        .then(() => {
          // 刷新表格数据并重载
          this.data = [];
          this.parentId = 0;
          this.$refs.crud.refreshTable();
          this.$refs.crud.toggleSelection();
          // 表格数据重载
          this.onLoad(this.page);
          this.$message({
            type: "success",
            message: "操作成功!"
          });
        });
    },
    searchReset() {
      this.query = {};
      this.parentId = 0;
      this.onLoad(this.page);
    },
    searchChange(params, done) {
      this.query = params;
      this.parentId = '';
      this.page.currentPage = 1;
      this.onLoad(this.page, params);
      done();
    },
    clickRowChange(row) {
      this.$refs.crud.toggleSelection();
      this.selectionList = row;
      this.$refs.crud.setCurrentRow(row);
      this.$refs.crud.toggleRowSelection(row); //选中当前行
    },
    selectionChange(list) {
      this.selectionList = list;
      this.$refs.crud.setCurrentRow(this.selectionList[list.length - 1]);
    },
    selectionClear() {
      this.selectionList = [];
      this.$refs.crud.toggleSelection();
    },
    beforeOpen(done, type) {
      if (["add", "edit"].includes(type)) {
        this.initData();
      }
      if (["edit", "view"].includes(type)) {
        getMenu(this.form.id).then(res => {
          this.form = res.data.data;
        });
      }
      done();
    },
    beforeClose(done) {
      this.parentId = "";
      const column = this.findObject(this.option.column, "parentId");
      column.value = "";
      column.addDisabled = false;
      done();
    },
    currentChange(currentPage) {
      this.page.currentPage = currentPage;
    },
    sizeChange(pageSize) {
      this.page.pageSize = pageSize;
    },
    refreshChange() {
      this.onLoad(this.page, this.query);
    },
    onLoad(page, params = {}) {
      this.loading = true;
      getLazyList(this.parentId, Object.assign(params, this.query)).then(res => {
        this.data = res.data.data;
        this.loading = false;
        this.selectionClear();
      });
    },
    treeLoad(tree, treeNode, resolve) {
      const parentId = tree.id;
      getLazyList(parentId).then(res => {
        resolve(res.data.data);
      });
    }
  }
};
</script>

<style>
</style>
